﻿using System;
using System.Drawing;
using System.Windows.Forms;
//using System.Collections.Generic;
//using System.ComponentModel;
//using System.Data;
//using System.Text;
// Библиотеки для работы с графикой;
using Tao.OpenGl;
using Tao.Platform.Windows;
using Tao.FreeGlut;

namespace TAO_Fullscreen_Mode
{
    public partial class FormFS : Form
    {
        #region +++ GLOBAL +++
        // Флаг полноэкранного режима;
        private bool FS;
        #endregion

        #region CONSTRUCTORS
        public FormFS()
        {
            InitializeComponent();
            // Инициализируем работу TaoWin;
            TaoWin.InitializeContexts();
            // Инициализация OpenGL
            // !!!Последовательность строк имеет значение!!!
            InitGL();
            ResizeGlScene();
            DrawGlScene();
        }

        public FormFS(bool fullscreen)
        {
            InitializeComponent();
            // Инициализируем работу TaoWin;
            TaoWin.InitializeContexts();
            // Инициализация OpenGL
            // !!!Последовательность строк имеет значение!!!
            InitGL();
            ResizeGlScene();
            DrawGlScene();
        }
        #endregion

        #region Инициализация OpenGL
        private void InitGL()
        {
            Glut.glutInit();
            Glut.glutInitDisplayMode(Glut.GLUT_RGBA |
                Glut.GLUT_DEPTH |
                Glut.GLUT_DOUBLE);
            // Разрешить плавное цветовое сглаживание;
            Gl.glShadeModel(Gl.GL_SMOOTH);
            // Разрешить тест глубины;
            Gl.glEnable(Gl.GL_DEPTH_TEST);
            // Разрешить очистку буфера глубины;
            Gl.glClearDepth(1.0f);
            // Определение типа теста глубины;
            Gl.glDepthFunc(Gl.GL_LEQUAL);
            // Слегка улучшим вывод перспективы;
            Gl.glHint(Gl.GL_PERSPECTIVE_CORRECTION_HINT, Gl.GL_NICEST);
            // Разрешаем смешивание;
            Gl.glEnable(Gl.GL_BLEND);
            // Устанавливаем тип смешивания;
            Gl.glBlendFunc(Gl.GL_SRC_ALPHA, Gl.GL_ONE_MINUS_SRC_ALPHA);
        }
        #endregion

        #region Изменение размеров окна OpenGL
        void ResizeGlScene()
        {
            // Предупредим деление на нуль;
            if (TaoWin.Height == 0)
            {
                TaoWin.Height = 1;
            }
            // Сбрасываем текущую область просмотра;
            Gl.glViewport(0, 0, TaoWin.Width, TaoWin.Height);
            // Выбираем матрицу проекций;
            Gl.glMatrixMode(Gl.GL_PROJECTION);
            // Сбрасываем выбранную матрицу;
            Gl.glLoadIdentity();
            // Вычисляем новые геометрические размеры сцены;
            Glu.gluPerspective(45.0f, (float)TaoWin.Width / (float)TaoWin.Height, 0.1f, 100.0f);
            // Выбираем матрицу вида модели;
            Gl.glMatrixMode(Gl.GL_MODELVIEW);
            // Сбрасываем ее;
            Gl.glLoadIdentity();
        }

        // Обработчик события изменения размеров формы;
        private void FormFS_Resize(object sender, EventArgs e)
        {
            // Вначале изменим размеры сцены;
            ResizeGlScene();
            // А потом перерисуем;
            DrawGlScene();
        }
        #endregion

        #region РИСОВАНИЕ СЦЕНЫ
        private void DrawGlScene()
        {
            // Выбираем цвет очистки экрана;
            Gl.glClearColor(0.0f, 0.8f, 0.0f, 1.0f);
            // Очищаем экран выбранным цветом;
            Gl.glClear(Gl.GL_COLOR_BUFFER_BIT | Gl.GL_DEPTH_BUFFER_BIT);
            // Сбрасываем текущую матрицу проекций;
            Gl.glLoadIdentity();
            // Обязательно нужно сместиться по оси Z,
            // иначе ни чего не будет видно;
            Gl.glTranslatef(0, 0, -3.0f);

            // Помещаем матрицу в стек, чтобы не потерять
            // положение камеры в пространстве. 
            // Извлечем ее перед перерисовкой окна
            // используя Gl.glPopMatrix();
            Gl.glPushMatrix();
            Glu.GLUquadric disk = Glu.gluNewQuadric();
            Glu.gluQuadricDrawStyle(disk, Glu.GLU_FILL);
            Gl.glColor3ub(255, 0, 150);
            Glu.gluDisk(disk, 0.5, 1.0, 30, 1);
            Gl.glPopMatrix();

            Gl.glFlush();
            TaoWin.Invalidate();
        }
        #endregion

        #region Смена режима (Fullscreen/Window)
        // Смена режима (Fullscreen/Window)
        private void ScreenMode(bool fullscreen)
        {
            // Присваиваем значение "глобальной" переменной;
            FS = fullscreen;

            if (FS)
            {   // *** ПОЛНОЭКРАННЫЙ РЕЖИМ ***
                // Скрываем рамку окна;
                this.FormBorderStyle = FormBorderStyle.None;
                // Разворачиваем окно;
                this.WindowState = FormWindowState.Maximized;
                // --- Не обязательный пункт --- 
                // Делаем курсор в форме руки;
                // TaoWin.Cursor = Cursors.Hand;
            }
            else
            {   // *** ОКОННЫЙ РЕЖИМ ***
                // Возвращаем состояние окна;
                this.WindowState = FormWindowState.Normal;
                // Показываем масштабируемую рамку окна;
                this.FormBorderStyle = FormBorderStyle.Sizable;
                // --- Не обязательный пункт ---
                // Возвращаем курсор по-умолчанию;
                // TaoWin.Cursor = Cursors.Default;
                // Задаем размеры окна;
                this.Width = 400;   // Ширина;
                this.Height = 300;  // Высота;
            }
            ResizeGlScene();
        }
        #endregion  

        #region События нажатий клавиш клавиатуры
        // События нажатий клавиш клавиатуры; 
        private void FormFS_KeyUp(object sender, KeyEventArgs e)
        {
            #region EXIT
            // Нажата клавиша ЕSCAPE;
            if (e.KeyCode == Keys.Escape)
            {
                // Завершим приложение;
                this.Close();
            }
            #endregion

            #region Полноэкранный или оконный режим
            // Нажата клавиша F2;
            if (e.KeyCode == Keys.F2)
            {
                // Изменяем режим на ПОЛНОЭКРАННЫЙ;
                ScreenMode(true);
                // Прячем курсор;
                Cursor.Hide();
            }
            // Нажата клавиша F1;
            if (e.KeyCode == Keys.F1)
            {
                // Изменяем режим на оконный;
                ScreenMode(false);
                // Показываем курсор;
                Cursor.Show();
            }
            #endregion
        }
        #endregion

        private void FormFS_Load(object sender, EventArgs e)
        {

        }
    }
}
